#PBS -l nodes=1:ppn=16:xeon16
#PBS -l walltime=0:25:0
#PBS -o stdout.log
#PBS -e stderr.log

import os
import sys
from glob import glob
import numpy as np
from ase.visualize import view
from ase.io import read, write
from gpaw import GPAW, PoissonSolver, FermiDirac
from gpaw.utilities import h2gpts
from gpaw.xc.libvdwxc import vdw_df
from gpaw.mpi import world

cwd = os.getcwd()
path0, mydirname = os.path.split(cwd)
molno = int(mydirname.split('.')[1])
fnames = glob('../xyz_displaced/s%02d_??.xyz' % molno)
fnames.sort()

configs = [read(fname) for fname in fnames]

atoms = configs[-1]
atoms.pbc = 1
tmp = atoms.copy()
tmp.center(vacuum=10.0)
atoms.cell = tmp.cell # Atoms remain in corner (lower egg-box)

d1 = configs[1].positions[-1]
d2 = configs[2].positions[-1]
dx1 = d2 - d1
DX = configs[2].positions - configs[1].positions

i1 = [i for i, pos in enumerate(DX) if np.vdot(dx1, DX[i]) > 0]
i2 = [i for i, pos in enumerate(DX) if np.vdot(dx1, DX[i]) <= 0]

mol1 = atoms[i1]
mol2 = atoms[i2]
assert len(mol1) + len(mol2) == len(atoms)

def getcalc(**kwargs):
    kwargs1 = dict(gpts=h2gpts(0.17, atoms.cell, idiv=8),
                   mode='lcao',
                   basis='dzp',
                   convergence=dict(density=2e-6),
                   occupations=FermiDirac(0.0),
                   xc=vdw_df(),
                   poissonsolver=PoissonSolver(relax='GS', eps=1e-12))
    kwargs1.update(kwargs)
    calc = GPAW(**kwargs1)
    return calc

atoms.calc = getcalc(txt='gpaw.s22.txt')
mol1.calc = getcalc(txt='gpaw.mol1.txt')
mol2.calc = getcalc(txt='gpaw.mol2.txt')

for i, config in enumerate(configs):
    atoms.positions[:] = config.positions
    id = '%03d' % i

    atoms.calc.set(txt='gpaw.s22.%s.txt' % id)
    mol1.calc.set(txt='gpaw.mol1.%s.txt' % id)
    mol2.calc.set(txt='gpaw.mol2.%s.txt' % id)

    resultsfname = 'results.%s.txt' % id
    do_continue = 0
    if world.rank == 0:
        if os.path.exists(resultsfname):
            do_continue = 1
    do_continue = world.sum(do_continue)
    if do_continue:
        continue

    if world.rank == 0:
        fd = open(resultsfname, 'w')

    E = atoms.get_potential_energy()
    F0 = atoms.get_forces()

    mol1.positions[:] = atoms.positions[i1]
    mol2.positions[:] = atoms.positions[i2]
    Eref1 = mol1.get_potential_energy()
    Eref2 = mol2.get_potential_energy()
    Fref1 = mol1.get_forces()
    Fref2 = mol2.get_forces()

    write('out.s22.%s.traj' % id, atoms)
    write('out.mol1.%s.traj' % id, mol1)
    write('out.mol2.%s.traj' % id, mol2)
    x = np.linalg.norm(mol2.get_center_of_mass() - mol1.get_center_of_mass())

    if world.rank == 0:
        fd.write('%15.10f %15.10f\n' % (x, E - Eref1 - Eref2))
        fd.close()
